/*

Boost Software License - Version 1.0 - August 17th, 2003

Permission is hereby granted, free of charge, to any person or organization
obtaining a copy of the software and accompanying documentation covered by
this license (the "Software") to use, reproduce, display, distribute,
execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to
do so, all subject to the following:

The copyright notices in the Software and this entire statement, including
the above license grant, this restriction and the following disclaimer,
must be included in all copies of the Software, in whole or in part, and
all derivative works of the Software, unless such copies or derivative
works are solely in the form of machine-executable object code generated by
a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.

*/
module derelict.theora.theoradec;

private {
    import core.stdc.config;
    import derelict.util.loader;
    import derelict.util.system;

    static if( Derelict_OS_Windows )
        enum libNames = "theoradec.dll, libtheoradec-0.dll, libtheoradec.dll";
    else static if( Derelict_OS_Mac )
        enum libNames = "libtheoradec.dylib, libtheoradec.0.dylib";
    else static if( Derelict_OS_Posix )
        enum libNames = "libtheoradec.so, libtheoradec.so.0";
    else
        static assert( 0, "Need to implement libtheoradec libnames for this operating system." );
}

public {
    import derelict.ogg.ogg;
    import derelict.theora.codec;
}

enum {
    TH_DECCTL_GET_PPLEVEL_MAX = 1,
    TH_DECCTL_SET_PPLEVEL = 3,
    TH_DECCTL_SET_GRANPOS = 5,
    TH_DECCTL_SET_STRIPE_CB = 7,
    TH_DECCTL_SET_TELEMETRY_MBMODE = 9,
    TH_DECCTL_SET_TELEMETRY_MV = 11,
    TH_DECCTL_SET_TELEMETRY_QI = 13,
    TH_DECCTL_SET_TELEMETRY_BITS = 15,
}

extern( C ) nothrow alias th_stripe_decoded_func = void function( void*, ref th_ycbcr_buffer, int, int );

struct th_stripe_callback {
    void* ctx;
    th_stripe_decoded_func stripe_decoded;
}

struct th_dec_ctx;
struct th_setup_info;


extern( C ) @nogc nothrow {
    alias da_th_decode_headerin = int function( th_info*, th_comment*, th_setup_info**, ogg_packet* );
    alias da_th_decode_alloc = th_dec_ctx* function( const( th_info )*, const( th_setup_info )* );
    alias da_th_setup_free = void function( th_setup_info* );
    alias da_th_decode_ctl = int function( th_dec_ctx*, int, void*, size_t );
    alias da_th_decode_packetin = int function( th_dec_ctx*, const( ogg_packet )*, ogg_int64_t* );
    alias da_th_decode_ycbcr_out = int function( th_dec_ctx*, ref th_ycbcr_buffer );
    alias da_th_decode_free = void function( th_dec_ctx* );
}

__gshared {
    da_th_decode_headerin th_decode_headerin;
    da_th_decode_alloc th_decode_alloc;
    da_th_setup_free th_setup_free;
    da_th_decode_ctl th_decode_ctl;
    da_th_decode_packetin th_decode_packetin;
    da_th_decode_ycbcr_out th_decode_ycbcr_out;
    da_th_decode_free th_decode_free;
}

class DerelictTheoraDecLoader : SharedLibLoader {
    public this() {
        super( libNames );
    }

    protected override void loadSymbols() {
        bindFunc( cast( void** )&th_decode_headerin, "th_decode_headerin" );
        bindFunc( cast( void** )&th_decode_alloc, "th_decode_alloc" );
        bindFunc( cast( void** )&th_setup_free, "th_setup_free" );
        bindFunc( cast( void** )&th_decode_ctl, "th_decode_ctl" );
        bindFunc( cast( void** )&th_decode_packetin, "th_decode_packetin" );
        bindFunc( cast( void** )&th_decode_ycbcr_out, "th_decode_ycbcr_out" );
        bindFunc( cast( void** )&th_decode_free, "th_decode_free" );
    }
}

__gshared DerelictTheoraDecLoader DerelictTheoraDec;

shared static this() {
    DerelictTheoraDec = new DerelictTheoraDecLoader();
}