/*

Boost Software License - Version 1.0 - August 17th, 2003

Permission is hereby granted, free of charge, to any person or organization
obtaining a copy of the software and accompanying documentation covered by
this license (the "Software") to use, reproduce, display, distribute,
execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to
do so, all subject to the following:

The copyright notices in the Software and this entire statement, including
the above license grant, this restriction and the following disclaimer,
must be included in all copies of the Software, in whole or in part, and
all derivative works of the Software, unless such copies or derivative
works are solely in the form of machine-executable object code generated by
a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.

*/
module derelict.theora.theoraenc;

private {
    import core.stdc.config;
    import derelict.util.loader;
    import derelict.util.system;

    static if( Derelict_OS_Windows )
        enum libNames = "theoraenc.dll, libtheoraenc-1.dll, libtheoraenc.dll";
    else static if( Derelict_OS_Mac )
        enum libNames = "libtheoraenc.dylib, libtheoraenc.1.dylib";
    else static if( Derelict_OS_Posix )
        enum libNames = "libtheoraenc.so, libtheoraenc.so.1";
    else
        static assert( 0, "Need to implement libtheoraenc libnames for this operating system." );
}

public {
    import derelict.ogg.ogg;
    import derelict.theora.codec;
}

enum {
    TH_ENCCTL_SET_HUFFMAN_CODES = 0,
    TH_ENCCTL_SET_QUANT_PARAMS = 2,
    TH_ENCCTL_SET_KEYFRAME_FREQUENCY_FORCE = 4,
    TH_ENCCTL_SET_VP3_COMPATIBLE = 10,
    TH_ENCCTL_GET_SPLEVEL_MAX = 12,
    TH_ENCCTL_SET_SPLEVEL = 14,
    TH_ENCCTL_GET_SPLEVEL = 16,
    TH_ENCCTL_SET_DUP_COUNT = 18,
    TH_ENCCTL_SET_RATE_FLAGS = 20,
    TH_ENCCTL_SET_RATE_BUFFER = 22,
    TH_ENCCTL_2PASS_OUT = 24,
    TH_ENCCTL_2PASS_IN = 26,
    TH_ENCCTL_SET_QUALITY = 28,
    TH_ENCCTL_SET_BITRATE = 30,
    TH_RATECTL_DROP_FRAMES = 0x1,
    TH_RATECTL_CAP_OVERFLOW = 0x2,
    TH_RATECTL_CAP_UNDERFLOW = 0x4,
}

struct th_enc_ctx;

extern( C ) @nogc nothrow {
    alias da_th_encode_alloc = th_enc_ctx* function( th_info* );
    alias da_th_encode_ctl = int function( th_enc_ctx*, int, void*, size_t );
    alias da_th_encode_flushheader = int function( th_enc_ctx*, th_comment*, ogg_packet* );
    alias da_th_encode_ycbcr_in = int function( th_enc_ctx*, ref th_ycbcr_buffer );
    alias da_th_encode_packetout = int function( th_enc_ctx*, int, ogg_packet* );
    alias da_th_encode_free = void function( th_enc_ctx* );
}

__gshared {
    da_th_encode_alloc th_encode_alloc;
    da_th_encode_ctl th_encode_ctl;
    da_th_encode_flushheader th_encode_flushheader;
    da_th_encode_ycbcr_in th_encode_ycbcr_in;
    da_th_encode_packetout th_encode_packetout;
    da_th_encode_free th_encode_free;
}

class DerelictTheoraEncLoader : SharedLibLoader {
    public this() {
        super( libNames );
    }

    protected override void loadSymbols() {
        bindFunc( cast( void** )&th_encode_alloc, "th_encode_alloc" );
        bindFunc( cast( void** )&th_encode_ctl, "th_encode_ctl" );
        bindFunc( cast( void** )&th_encode_flushheader, "th_encode_flushheader" );
        bindFunc( cast( void** )&th_encode_ycbcr_in, "th_encode_ycbcr_in" );
        bindFunc( cast( void** )&th_encode_packetout, "th_encode_packetout" );
        bindFunc( cast( void** )&th_encode_free, "th_encode_free" );
    }
}

__gshared DerelictTheoraEncLoader DerelictTheoraEnc;

shared static this() {
    DerelictTheoraEnc = new DerelictTheoraEncLoader();
}